@@ -19,12 +19,6 @@ def find(app, env_name):
         else [root_env_name]
     )
 
-    for env_name in environments:
-        environment = app.get_environment(env_name)
-
-        try:
-            environment.check_compatibility()
-        except Exception:  # noqa: S112
-            continue
-
+    for env in environments:
+        environment = app.get_environment(env)
         app.display(environment.find())
