@@ -28,20 +28,9 @@ def shell(app, shell_name, shell_path, shell_args):  # no cov
         shell_args = app.config.shell.args
 
     if not shell_path:
-        import shellingham
-
-        try:
-            shell_name, command = shellingham.detect_shell()
-        except shellingham.ShellDetectionFailure:
-            from hatch.utils.fs import Path
-
-            shell_path = app.platform.default_shell
-            shell_name = Path(shell_path).stem
-        else:
-            if app.platform.windows:
-                shell_path = command
-            else:
-                shell_path, *shell_args = app.platform.modules.shlex.split(command)
+        shell_name, shell_path = app.shell_data
+        if not app.platform.windows:
+            shell_path, *shell_args = app.platform.modules.shlex.split(shell_path)
 
     with app.project.location.as_cwd():
         environment = app.get_environment()
