@@ -1,4 +1,5 @@
 from hatch.env.collectors.plugin.interface import EnvironmentCollectorInterface
+from hatch.env.internal import get_internal_env_config
 from hatch.env.utils import ensure_valid_environment
 
 
@@ -6,7 +7,7 @@ class DefaultEnvironmentCollector(EnvironmentCollectorInterface):
     PLUGIN_NAME = 'default'
 
     def get_initial_config(self):  # noqa: PLR6301
-        config = {}
-        ensure_valid_environment(config)
+        default_config = {}
+        ensure_valid_environment(default_config)
 
-        return {'default': config}
+        return {'default': default_config, **get_internal_env_config()}
